function DragManager(source, container)
{
	if (DragManager.instance)
	{
		DragManager.instance.destroy();
	}
	
	this.__defineGetter__('targets', function(){
		return typeof source['getDropTargets'] == 'function' ? source.getDropTargets() : [];
	});
	
	this.source = source;
	$(this.source.DOM).addClass('drop-source');
	
	this.initHelper(container || document.body);
	
	this.highlightTargets();
	
	DragManager.instance = this;
	
	this.callback('onDragStart', this.source);
	this.refreshTargetCache();
	this.targetIn(source);
}

DragManager.instance = null;
DragManager.position = {x:0, y:0};
DragManager.helperOpacity = 0.9;
DragManager.startTimer = null;
DragManager.revertDelay = 30;

DragManager.prototype.source = null;
DragManager.prototype.helper = null;
DragManager.prototype.targets = null;
DragManager.prototype.targetCache = {};
DragManager.prototype.target = null;

DragManager.init = function()
{
	window.addEventListener(MOUSE_DOWN_EVENT, DragManager.handleEvent, false);
	window.addEventListener(MOUSE_MOVE_EVENT, DragManager.handleEvent, false);
	window.addEventListener(MOUSE_UP_EVENT, DragManager.handleEvent, true);
}

DragManager.handleEvent = function(e)
{
	DragManager.refreshPosition(e);
	if (DragManager.instance && !DragManager.instance.completed)
	{
		switch (e.type)
		{
			case MOUSE_MOVE_EVENT:
				DragManager.instance.drag();
				break;
				
			case MOUSE_UP_EVENT:
				DragManager.instance.stop();
				break;
		}

		e.stopImmediatePropagation();
		e.preventDefault();
		return false;
	}
}

DragManager.refreshPosition = function(e)
{
	var evt = e['originalEvent'] || e;
	var pos = evt['touches'] ? evt.touches[0] : evt;
	DragManager.position.x = pos ? pos.clientX : 0;
	DragManager.position.y = pos ? pos.clientY : 0;
}

DragManager.prototype.refreshTargetCache = function()
{
	var aabb;
	this.targetCache = {};
	for (var i = 0, len = this.targets.length; i < len; i++)
	{
		aabb = COM.AABB(this.targets[i]);
		if (aabb) 
		{
			this.targetCache[COM(this.targets[i]).ID] = aabb;
		}
	}
}

DragManager.prototype.callback = function(func, obj)
{
	if (!COM(obj) || (typeof COM(obj)[func] != 'function')) return;
	return COM(obj)[func](this);
}

DragManager.add = function(node)
{
	node = (node instanceof HTMLElement) ? node : node.DOM;
	LongTap.add(node, DragManager.startHandler);
}

DragManager.startHandler = function(e)
{
	if (DragManager.instance) return;
	
	var source = COM(e.target);
	if (typeof source['dragAllowed'] != 'undefined')
	{
		if (!source.dragAllowed || (typeof source['dragAllowed'] == 'function') && !source.dragAllowed()) 
		{
			return;
		}
	}
	iScroll.enabled = false;
	DragManager.start(source);
}

DragManager.remove = function(node)
{
	node = (node instanceof HTMLElement) ? node : node.DOM;
	LongTap.remove(node, DragManager.startHandler);
}

DragManager.start = function(source)
{
	var d = new DragManager(source);
	return d;
}

DragManager.prototype.targetIn = function(target)
{
	if (!target || (this.target && target.DOM.isSameNode(this.target.DOM))) return;

	this.targetOut();
	
	this.target = target;
	
	var helper = $(this.helper);
	helper.removeClass('helper-drop-denied')
		.addClass('helper-drop-allowed')
		//.transition({'scale': 1.0, opacity:1.0}, 0)
	;
		
	this.callback('onDragEnter', this.target);
}

DragManager.prototype.targetOut = function(target)
{
	target = target || this.target;
	if (!target) return;
	
	this.callback('onDragLeave', target);
	this.target = null;
	
	var helper = $(this.helper);
	helper.removeClass('helper-drop-allowed')
		.addClass('helper-drop-denied')
		//.transition({'scale': 1.0, opacity:DragManager.helperOpacity}, 0)
	;
}

DragManager.prototype.drag = function()
{
	this.moveHelper(DragManager.position.x, DragManager.position.y);
	
	var target = this.filterActiveTarget(DragManager.position.x, DragManager.position.y);
	if (target)
	{
		this.targetIn(target);
	}
	else
	{
		this.targetOut();
	}
	return false;
}

DragManager.prototype.stop = function()
{
	DragManager.targetCache = {};
	this.accept();
	return false;
}

DragManager.topZIndex = function(container)
{
	var zIndex = 1;
	for (var i = 0, len = container.children.length; i < len; i++) 
	{
		var idx = parseInt(window.getComputedStyle(container.children[i]).zIndex);
		if (isNaN(idx)) idx = i+0;
		zIndex = Math.max(zIndex, idx+1);
	}
	return zIndex;
}

DragManager.prototype.initHelper = function(container)
{
	if (typeof this.source['createDragHelper'] != 'function')
	{
		this.helper = this.source.DOM.cloneNode();
		this.helper.removeAttribute('id');
	}
	else
	{
		this.helper = this.source.createDragHelper();
	}
	if (!this.helper) 
	{
		this.helper = document.createElement('drag-helper');
	}

	container = container || document.body;
	container.appendChild(this.helper);
	
	$(this.helper).css({
		'position': 'absolute',
		'z-index': DragManager.topZIndex(container),
		'pointer-events': 'none',
	});
	
	//$(this.helper).transition({'opacity': DragManager.helperOpacity});	
	
	this.moveHelper(DragManager.position.x, DragManager.position.y);
	this.helper.initialPosition = $(this.helper).offset();
	$(this.helper).addClass('drag-helper helper-drop-denied');
}

DragManager.prototype.moveHelper = function(x, y)
{
	if (!this.helper) return;
	$(this.helper).css({
		'left': (x - this.helper.offsetWidth*0.5) + 'px',
		'top': (y - 80) + 'px',
	});
}

DragManager.prototype.highlightTargets = function()
{
	var targets = this.targets;
	for (var i = 0, len = targets.length; i < len; i++)
	{
		var target = this.targets[i];
		this.callback('onDragTarget', target);
	}
}

DragManager.prototype.filterActiveTarget = function(x, y)
{
	var target, aabb;
	for (var i = 0, len = this.targets.length; i < len; i++)
	{
		target = COM(this.targets[i]) || {DOM:this.targets[i]};
		aabb = this.targetCache[target.DOM.id];
		if (COM.hitTestPoint(target.DOM.id, x, y, aabb) && target['canDropHere'])
		{
			var accept = (typeof target['acceptsDrop'] == 'function') ? target.acceptsDrop(this.source) : true;
			if (accept) 
			{
				return target;
			}
		}
	}
	return null;
}

DragManager.prototype.revert = function()
{
	var h = this.helper;
	
	var revert = {
		x: h.initialPosition.left - $(h).offset().left,
		y: h.initialPosition.top - $(h).offset().top,
		opacity: 0
	}
	var t = Math.max(DragManager.revertDelay, DragManager.revertDelay * (Math.sqrt(Math.pow(revert.x, 2) + Math.pow(revert.y, 2)) / 100));
	
	$(h).transition(revert, t, 'linear', $.proxy(this.destroy, this));
}

DragManager.prototype.accept = function()
{
	this.completed = true;
	
	var target = COM(this.target);
	if (!target || !target.canDropHere(this.source)) 
	{
		return this.revert();
	}
	
	this.callback('onDrop', target);
	$(this.helper).transition({scale:1.0, opacity: 0}, 100, 'linear', $.proxy(this.destroy, this));
}

DragManager.prototype.completed = false;
DragManager.prototype.destroy = function()
{
	iScroll.enabled = true;
	this.completed = true;
	
	this.callback('onDragEnd', this.source);
	var targets = this.targets;
	for (var i = 0, len = targets.length; i < len; i++)
	{
		var target = this.targets[i];
		this.callback('onDragEnd', COM(target));
	}
	
	if (COM(this.helper))
	{
		COM(this.helper).destroy();
	}
	else
	{
		$(this.helper).remove();
	}
	
	$(this.source.DOM).removeClass('drop-source');
	
	this.source = null;
	this.helper = null;
	this.targets = null;
	
	DragManager.instance = null;
}

